@@ -16,7 +16,7 @@ from gitpkg.errors import (
     AmbiguousDestinationError,
     CouldNotFindDestinationError,
     GitPkgError,
-    InvalidInstallMethodError,
+    PackageAlreadyInstalledError,
     PackageRootDirNotFoundError,
     UnknownPackageError,
 )
@@ -27,7 +27,7 @@ _COMMAND_PREFIX = "command_"
 
 
 class CLI:
-    _pm: PkgManager = None
+    _pkg_manager: PkgManager | None = None
     _args: list[str] = None
     _debug_mode: bool = False
 
@@ -46,11 +46,6 @@ class CLI:
                 handlers=[RichHandler()],
             )
 
-        try:
-            self._pm = PkgManager.from_environment()
-        except InvalidGitRepositoryError:
-            fatal("could not find a git repository")
-
         parser = argparse.ArgumentParser(
             description="A git powered package manager built on top of submodules.",
             usage=f"""git pkg <command> [<args>]
@@ -74,6 +69,9 @@ Commands:
             parser.print_help()
             fatal(f"Unrecognized command: {args.command}")
 
+        # reset package manager for every run
+        self._pkg_manager = None
+
         try:
             getattr(self, command_func)()
         except GitPkgError as err:
@@ -105,6 +103,18 @@ Commands:
 
         return commands_list
 
+    @property
+    def _pm(self):
+        """Initializes package manager only when needed"""
+        if self._pkg_manager:
+            return self._pkg_manager
+
+        try:
+            self._pkg_manager = PkgManager.from_environment()
+            return self._pkg_manager
+        except InvalidGitRepositoryError:
+            fatal("could not find a git repository")
+
     def _render_package_name(
         self,
         dest: Destination,
@@ -183,8 +193,8 @@ Commands:
             box=None,
         )
 
-        table.add_column("Name")
-        table.add_column("Path")
+        table.add_column("Name", overflow="fold")
+        table.add_column("Path", overflow="fold")
 
         for dest in self._pm.destinations():
             table.add_row(dest.name, str(Path(dest.path).absolute()))
@@ -323,16 +333,12 @@ Commands:
         if args.package_root:
             package_root = args.package_root
 
-        if args.install_method and args.install_method not in ["link"]:
-            raise InvalidInstallMethodError(args.install_method)
-
         pkg = PkgConfig(
             name=name,
             url=args.repository_url,
             package_root=package_root,
             updates_disabled=args.disable_updates,
             branch=args.branch,
-            install_method=args.install_method,
         )
 
         if not self._pm.is_package_registered(dest, pkg):
@@ -370,10 +376,10 @@ Commands:
             box=None,
         )
 
-        table.add_column("Name")
-        table.add_column("Install Dir")
-        table.add_column("Hash")
-        table.add_column("Last Update")
+        table.add_column("Name", overflow="fold")
+        table.add_column("Install Dir", overflow="fold")
+        table.add_column("Hash", overflow="fold")
+        table.add_column("Last Update", overflow="fold")
 
         found_one = False
 
@@ -447,12 +453,9 @@ Commands:
 
                     found_any = True
 
-                    has_pkg_changed = self._pm.has_package_config_been_changed(
-                        dest, pkg
-                    )
-                    already_installed = self._pm.is_package_installed(dest, pkg)
-
-                    if already_installed and not has_pkg_changed:
+                    try:
+                        self._pm.install_package(dest, pkg)
+                    except PackageAlreadyInstalledError:
                         pkg_name = self._render_package_name(dest, pkg)
                         tree.add(
                             f"{pkg_name} is already installed.",
@@ -461,13 +464,13 @@ Commands:
                         )
                         continue
 
-                    self._pm.install_package(dest, pkg)
-
                     pkg_name = self._render_package_name(dest, pkg)
                     tree.add(f"{pkg_name} has been installed.")
+
             if found_any:
                 console.print(tree)
                 return
+
             console.print("No packages were installed.")
 
     def command_update(self):
@@ -527,18 +530,27 @@ Commands:
 
         tree = Tree("Package update results:")
 
+        updated_packages: dict[str, PkgUpdateResult] = {}
+
         with console.status("[bold green]Updating packages...") as status:
             for dest, pkg in to_install:
-                pkg_name = self._render_package_name(dest, pkg)
-                status.update(f"[bold green]Updating {pkg_name}...")
+                pkg_ident = self._pm.package_identifier(dest, pkg)
 
-                stats_before_update = self._pm.package_stats(dest, pkg)
-                update_result = self._pm.update_package(
-                    dest,
-                    pkg,
-                    discard_untracked_changes=args.force,
-                    check_only=args.check,
-                )
+                # there is no need to update a repo again if two
+                # packages share it
+                if pkg_ident not in updated_packages:
+                    pkg_name = self._render_package_name(dest, pkg)
+                    status.update(f"[bold green]Updating {pkg_name}...")
+
+                    stats_before_update = self._pm.package_stats(dest, pkg)
+                    updated_packages[pkg_ident] = self._pm.update_package(
+                        dest,
+                        pkg,
+                        discard_untracked_changes=args.force,
+                        check_only=args.check,
+                    )
+
+                update_result = updated_packages[pkg_ident]
 
                 pkg_name = self._render_package_name(dest, pkg)
 
